@@ -38,15 +38,17 @@
 
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
+#include <gconf/gconf-client.h>
 
-#include <libhal-gdevice.h>
-#include <libhal-gmanager.h>
-#include <libdbus-proxy.h>
+#include <hal-device.h>
+#include <hal-manager.h>
+
+#include "egg-dbus-proxy.h"
+#include "egg-debug.h"
+#include "egg-discrete.h"
 
 #include "gpm-brightness-kbd.h"
-#include "gpm-conf.h"
 #include "gpm-common.h"
-#include "gpm-debug.h"
 #include "gpm-light-sensor.h"
 #include "gpm-marshal.h"
 
@@ -56,6 +58,7 @@
 
 struct GpmBrightnessKbdPrivate
 {
+	gboolean		 has_hw;
 	gboolean		 does_own_updates;	/* keys are hardwired */
 	gboolean		 does_own_dimming;	/* hardware auto-fades */
 	gboolean		 is_dimmed;
@@ -65,9 +68,9 @@ struct GpmBrightnessKbdPrivate
 	guint			 level_std_hw;
 	guint			 levels;
 	gchar			*udi;
-	GpmConf			*conf;
+	GConfClient		*conf;
 	GpmLightSensor		*sensor;
-	DbusProxy		*gproxy;
+	EggDbusProxy		*gproxy;
 };
 
 enum {
@@ -76,7 +79,7 @@ enum {
 };
 
 G_DEFINE_TYPE (GpmBrightnessKbd, gpm_brightness_kbd, G_TYPE_OBJECT)
-static guint	     signals [LAST_SIGNAL] = { 0, };
+static guint	     signals [LAST_SIGNAL] = { 0 };
 
 /**
  * gpm_brightness_kbd_get_hw:
@@ -93,29 +96,32 @@ gpm_brightness_kbd_get_hw (GpmBrightnessKbd *brightness,
 	GError     *error = NULL;
 	gboolean    ret;
 	DBusGProxy *proxy;
+	gint brightness_level;
 
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
-		gpm_warning ("not connected to HAL");
+		egg_warning ("not connected to HAL");
 		return FALSE;
 	}
 
 	ret = dbus_g_proxy_call (proxy, "GetBrightness", &error,
 				 G_TYPE_INVALID,
-				 G_TYPE_UINT, brightness_level_hw,
+				 G_TYPE_INT, &brightness_level,
 				 G_TYPE_INVALID);
 	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
+		egg_debug ("ERROR: %s", error->message);
 		g_error_free (error);
 	}
-	if (ret == FALSE) {
+	if (!ret) {
 		/* abort as the DBUS method failed */
-		gpm_warning ("GetBrightness failed!");
+		egg_warning ("GetBrightness failed!");
 		return FALSE;
-	}
+	} 
+
+	*brightness_level_hw = (guint)brightness_level;
 	return TRUE;
 }
 
@@ -129,8 +135,7 @@ gpm_brightness_kbd_get_hw (GpmBrightnessKbd *brightness,
  * Return value: Success.
  **/
 static gboolean
-gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
-			   guint	     brightness_level_hw)
+gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness, guint brightness_level_hw)
 {
 	GError *error = NULL;
 	gboolean ret;
@@ -139,32 +144,31 @@ gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	proxy = dbus_proxy_get_proxy (brightness->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (brightness->priv->gproxy);
 	if (proxy == NULL) {
-		gpm_warning ("not connected to HAL");
+		egg_warning ("not connected to HAL");
 		return FALSE;
 	}
 
-	if (brightness_level_hw < 0 ||
-	    brightness_level_hw > brightness->priv->levels - 1) {
-		gpm_warning ("set outside range (%i of %i)",
+	if (brightness_level_hw > brightness->priv->levels - 1) {
+		egg_warning ("set outside range (%i of %i)",
 			     brightness_level_hw, brightness->priv->levels - 1);
 		return FALSE;
 	}
 
-	gpm_debug ("Setting %i of %i", brightness_level_hw, brightness->priv->levels - 1);
+	egg_debug ("Setting %i of %i", brightness_level_hw, brightness->priv->levels - 1);
 
 	ret = dbus_g_proxy_call (proxy, "SetBrightness", &error,
 				 G_TYPE_INT, brightness_level_hw,
 				 G_TYPE_INVALID,
 				 G_TYPE_INVALID);
 	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
+		egg_debug ("ERROR: %s", error->message);
 		g_error_free (error);
 	}
-	if (ret == FALSE) {
+	if (!ret) {
 		/* abort as the DBUS method failed */
-		gpm_warning ("SetBrightness failed!");
+		egg_warning ("SetBrightness failed!");
 		return FALSE;
 	}
 	brightness->priv->current_hw = brightness_level_hw;
@@ -179,9 +183,7 @@ gpm_brightness_kbd_set_hw (GpmBrightnessKbd *brightness,
  * Just do the step up and down, after knowing the step interval
  **/
 static gboolean
-gpm_brightness_kbd_dim_hw_step (GpmBrightnessKbd *brightness,
-				guint             new_level_hw,
-				guint		  step_interval)
+gpm_brightness_kbd_dim_hw_step (GpmBrightnessKbd *brightness, guint new_level_hw, guint step_interval)
 {
 	guint current_hw;
 	gint a;
@@ -190,16 +192,15 @@ gpm_brightness_kbd_dim_hw_step (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
 	current_hw = brightness->priv->current_hw;
-	gpm_debug ("new_level_hw=%i, current_hw=%i", new_level_hw, current_hw);
+	egg_debug ("new_level_hw=%i, current_hw=%i", new_level_hw, current_hw);
 
 	/* we do the step interval as we can have insane levels of brightness_kbd */
-	if (new_level_hw == current_hw) {
+	if (new_level_hw == current_hw)
 		return FALSE;
-	}
 
 	if (new_level_hw > current_hw) {
 		/* going up */
-		for (a=current_hw; a <= new_level_hw; a+=step_interval) {
+		for (a=current_hw; a <= (gint) new_level_hw; a+=step_interval) {
 			gpm_brightness_kbd_set_hw (brightness, a);
 			g_usleep (1000 * DIM_INTERVAL);
 		}
@@ -224,7 +225,7 @@ gpm_brightness_kbd_get_step (GpmBrightnessKbd *brightness)
 {
 	int step;
 
-	g_return_val_if_fail (brightness != NULL, FALSE);
+	g_return_val_if_fail (brightness != NULL, 0);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), 0);
 
 	if (brightness->priv->levels < 20) {
@@ -251,7 +252,7 @@ gpm_brightness_kbd_dim_hw (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	gpm_debug ("new_level_hw=%i", new_level_hw);
+	egg_debug ("new_level_hw=%i", new_level_hw);
 
 	/* some machines don't take kindly to auto-dimming */
 	if (brightness->priv->does_own_dimming) {
@@ -280,7 +281,12 @@ gpm_brightness_kbd_set_dim (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	level_hw = gpm_percent_to_discrete (brightness_level, brightness->priv->levels);
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
+	level_hw = egg_discrete_from_percent (brightness_level, brightness->priv->levels);
 
 	/* If the current brightness_kbd is less than the dim brightness_kbd then just
 	 * use the current brightness_kbd so that we don't *increase* in brightness_kbd
@@ -288,12 +294,12 @@ gpm_brightness_kbd_set_dim (GpmBrightnessKbd *brightness,
 	if (brightness->priv->level_std_hw > level_hw) {
 		brightness->priv->level_dim_hw = level_hw;
 	} else {
-		gpm_warning ("Current brightness_kbd is %i, dim brightness_kbd is %i.",
+		egg_warning ("Current brightness_kbd is %i, dim brightness_kbd is %i.",
 			     brightness->priv->level_std_hw, level_hw);
 		brightness->priv->level_dim_hw = brightness->priv->level_std_hw;
 	}
 	/* if in this state, then update */
-	if (brightness->priv->is_dimmed == TRUE) {
+	if (brightness->priv->is_dimmed) {
 		gpm_brightness_kbd_dim_hw (brightness, brightness->priv->level_dim_hw);
 	}
 	return TRUE;
@@ -313,7 +319,12 @@ gpm_brightness_kbd_set_std (GpmBrightnessKbd *brightness,
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	level_hw = gpm_percent_to_discrete (brightness_level,
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
+	level_hw = egg_discrete_from_percent (brightness_level,
 						 brightness->priv->levels);
 	brightness->priv->level_std_hw = level_hw;
 
@@ -336,9 +347,14 @@ gpm_brightness_kbd_dim (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	/* check to see if we are already dimmed */
-	if (brightness->priv->is_dimmed == TRUE) {
-		gpm_warning ("already dim'ed");
+	if (brightness->priv->is_dimmed) {
+		egg_warning ("already dim'ed");
 		return FALSE;
 	}
 	brightness->priv->is_dimmed = TRUE;
@@ -358,9 +374,14 @@ gpm_brightness_kbd_undim (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	/* check to see if we are already dimmed */
 	if (brightness->priv->is_dimmed == FALSE) {
-		gpm_warning ("already undim'ed");
+		egg_warning ("already undim'ed");
 		return FALSE;
 	}
 	brightness->priv->is_dimmed = FALSE;
@@ -382,10 +403,15 @@ gpm_brightness_kbd_get (GpmBrightnessKbd *brightness,
 {
 	guint percentage;
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
-	percentage = gpm_discrete_to_percent (brightness->priv->current_hw,
+	percentage = egg_discrete_to_percent (brightness->priv->current_hw,
 						       brightness->priv->levels);
 	*brightness_level = percentage;
 	return TRUE;
@@ -406,6 +432,11 @@ gpm_brightness_kbd_up (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	/* Do we find the new value, or set the new value */
 	if (brightness->priv->does_own_updates) {
 		gpm_brightness_kbd_get_hw (brightness, &brightness->priv->current_hw);
@@ -419,9 +450,9 @@ gpm_brightness_kbd_up (GpmBrightnessKbd *brightness)
 		gpm_brightness_kbd_set_hw (brightness, brightness->priv->current_hw + step);
 	}
 
-	percentage = gpm_discrete_to_percent (brightness->priv->current_hw,
+	percentage = egg_discrete_to_percent (brightness->priv->current_hw,
 					      brightness->priv->levels);
-	gpm_debug ("emitting brightness-changed (%i)", percentage);
+	egg_debug ("emitting brightness-changed (%i)", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_CHANGED], 0, percentage);
 
 	return TRUE;
@@ -442,6 +473,11 @@ gpm_brightness_kbd_down (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	/* Do we find the new value, or set the new value */
 	if (brightness->priv->does_own_updates) {
 		gpm_brightness_kbd_get_hw (brightness, &brightness->priv->current_hw);
@@ -449,36 +485,19 @@ gpm_brightness_kbd_down (GpmBrightnessKbd *brightness)
 		/* macbook pro has a bazzillion brightness_kbd levels, be a bit clever */
 		step = gpm_brightness_kbd_get_step (brightness);
 		/* don't underflow */
-		if (brightness->priv->current_hw < step) {
+		if ((gint) brightness->priv->current_hw < step)
 			step = brightness->priv->current_hw;
-		}
 		gpm_brightness_kbd_set_hw (brightness, brightness->priv->current_hw - step);
 	}
 
-	percentage = gpm_discrete_to_percent (brightness->priv->current_hw,
+	percentage = egg_discrete_to_percent (brightness->priv->current_hw,
 					      brightness->priv->levels);
-	gpm_debug ("emitting brightness-changed (%i)", percentage);
+	egg_debug ("emitting brightness-changed (%i)", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_CHANGED], 0, percentage);
 
 	return TRUE;
 }
 
-/**keyboard_backlight
- * gpm_brightness_kbd_constructor:
- **/
-static GObject *
-gpm_brightness_kbd_constructor (GType		  type,
-			        guint		  n_construct_properties,
-			        GObjectConstructParam *construct_properties)
-{
-	GpmBrightnessKbd      *brightness;
-	GpmBrightnessKbdClass *klass;
-	klass = GPM_BRIGHTNESS_KBD_CLASS (g_type_class_peek (GPM_TYPE_BRIGHTNESS_KBD));
-	brightness = GPM_BRIGHTNESS_KBD (G_OBJECT_CLASS (gpm_brightness_kbd_parent_class)->constructor
-			      		     (type, n_construct_properties, construct_properties));
-	return G_OBJECT (brightness);
-}
-
 /**
  * gpm_brightness_kbd_finalize:
  **/
@@ -490,18 +509,11 @@ gpm_brightness_kbd_finalize (GObject *object)
 	g_return_if_fail (GPM_IS_BRIGHTNESS_KBD (object));
 	brightness = GPM_BRIGHTNESS_KBD (object);
 
-	if (brightness->priv->udi != NULL) {
-		g_free (brightness->priv->udi);
-	}
-	if (brightness->priv->gproxy != NULL) {
+	g_free (brightness->priv->udi);
+	if (brightness->priv->gproxy != NULL)
 		g_object_unref (brightness->priv->gproxy);
-	}
-	if (brightness->priv->conf != NULL) {
-		g_object_unref (brightness->priv->conf);
-	}
-	if (brightness->priv->sensor != NULL) {
+	if (brightness->priv->sensor != NULL)
 		g_object_unref (brightness->priv->sensor);
-	}
 
 	g_return_if_fail (brightness->priv != NULL);
 	G_OBJECT_CLASS (gpm_brightness_kbd_parent_class)->finalize (object);
@@ -514,20 +526,16 @@ static void
 gpm_brightness_kbd_class_init (GpmBrightnessKbdClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	object_class->finalize	   = gpm_brightness_kbd_finalize;
-	object_class->constructor  = gpm_brightness_kbd_constructor;
+	object_class->finalize = gpm_brightness_kbd_finalize;
 
 	signals [BRIGHTNESS_CHANGED] =
 		g_signal_new ("brightness-changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmBrightnessKbdClass, brightness_changed),
-			      NULL,
-			      NULL,
+			      NULL, NULL,
 			      g_cclosure_marshal_VOID__UINT,
-			      G_TYPE_NONE,
-			      1,
-			      G_TYPE_UINT);
+			      G_TYPE_NONE, 1, G_TYPE_UINT);
 
 	g_type_class_add_private (klass, sizeof (GpmBrightnessKbdPrivate));
 }
@@ -577,11 +585,11 @@ adjust_kbd_brightness_according_to_ambient_light (GpmBrightnessKbd *brightness,
 		state = STATE_FORCED_UNKNOWN;
 	}
 
- 	gpm_debug ("ambient light percent = %d", ambient_light);
+ 	egg_debug ("ambient light percent = %d", ambient_light);
 
 	if (state == STATE_FORCED_UNKNOWN) {
 		/* if this is the first time we're launched with ambient light data... */
-		if (ambient_light < 50 ) {
+		if (ambient_light < 50) {
 			gpm_brightness_kbd_set_std (brightness, 100);
 			state = STATE_FORCED_ON;
 		} else {
@@ -589,7 +597,7 @@ adjust_kbd_brightness_according_to_ambient_light (GpmBrightnessKbd *brightness,
 			state = STATE_FORCED_OFF;
 		}
 	} else {
-		if (ambient_light < 30 && state != STATE_FORCED_ON ) {
+		if (ambient_light < 30 && state != STATE_FORCED_ON) {
 			/* if it's dark.. and we haven't already turned light on...
 			 *   => turn it on.. full blast! */
 			gpm_brightness_kbd_set_std (brightness, 100);
@@ -662,6 +670,11 @@ gpm_brightness_kbd_toggle (GpmBrightnessKbd *brightness)
 	g_return_val_if_fail (brightness != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_BRIGHTNESS_KBD (brightness), FALSE);
 
+	if (!brightness->priv->has_hw) {
+		egg_debug ("no hardware");
+		return FALSE;
+	}
+
 	if (brightness->priv->is_disabled == FALSE) {
 		/* go dark, that's what the user wants */
 		gpm_brightness_kbd_set_std (brightness, 0);
@@ -687,12 +700,21 @@ static void
 gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 {
 	gchar **names;
-	HalGManager *manager;
-	HalGDevice *device;
+	HalManager *manager;
+	HalDevice *device;
+	DBusGConnection *connection;
 
 	brightness->priv = GPM_BRIGHTNESS_KBD_GET_PRIVATE (brightness);
 
-	brightness->priv->conf = gpm_conf_new ();
+	manager = hal_manager_new ();
+
+	/* save udi of kbd adapter */
+	hal_manager_find_capability (manager, "keyboard_backlight", &names, NULL);
+	if (names == NULL || names[0] == NULL) {
+		egg_warning ("No devices of capability keyboard_backlight");
+		brightness->priv->has_hw = FALSE;
+		goto out;
+	}
 
 	/* listen for ambient light changes.. if we have an ambient light sensor */
 	brightness->priv->sensor = gpm_light_sensor_new ();
@@ -701,35 +723,23 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 				  G_CALLBACK (sensor_changed_cb), brightness);
 	}
 
-	/* save udi of kbd adapter */
-	manager = hal_gmanager_new ();
-	hal_gmanager_find_capability (manager, "keyboard_backlight", &names, NULL);
-	g_object_unref (manager);
-	if (names == NULL || names[0] == NULL) {
-		gpm_warning ("No devices of capability keyboard_backlight");
-		return;
-	}
-
 	/* We only want first keyboard_backlight object (should only be one) */
 	brightness->priv->udi = g_strdup (names[0]);
-	hal_gmanager_free_capability (names);
-
 	brightness->priv->does_own_dimming = FALSE;
 	brightness->priv->does_own_updates = FALSE;
 	brightness->priv->is_disabled = FALSE;
+	brightness->priv->has_hw = TRUE;
 
 	/* get a managed proxy */
-	brightness->priv->gproxy = dbus_proxy_new ();
-	dbus_proxy_assign (brightness->priv->gproxy,
-			  DBUS_PROXY_SYSTEM,
-			  HAL_DBUS_SERVICE,
-			  brightness->priv->udi,
-			  HAL_DBUS_INTERFACE_KBD_BACKLIGHT);
+	brightness->priv->gproxy = egg_dbus_proxy_new ();
+	connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
+	egg_dbus_proxy_assign (brightness->priv->gproxy, connection, HAL_DBUS_SERVICE,
+			       brightness->priv->udi, HAL_DBUS_INTERFACE_KBD_BACKLIGHT);
 
 	/* get levels that the adapter supports -- this does not change ever */
-	device = hal_gdevice_new ();
-	hal_gdevice_set_udi (device, brightness->priv->udi);
-	hal_gdevice_get_uint (device, "keyboard_backlight.num_levels",
+	device = hal_device_new ();
+	hal_device_set_udi (device, brightness->priv->udi);
+	hal_device_get_uint (device, "keyboard_backlight.num_levels",
 			      &brightness->priv->levels, NULL);
 	g_object_unref (device);
 
@@ -740,39 +750,15 @@ gpm_brightness_kbd_init (GpmBrightnessKbd *brightness)
 	brightness->priv->level_dim_hw = 1;
 	brightness->priv->level_std_hw = 1;
 
-	gpm_debug ("Starting: (%i of %i)",
+	egg_debug ("Starting: (%i of %i)",
 		   brightness->priv->current_hw,
 		   brightness->priv->levels - 1);
 
 	/* choose a start value */
 	adjust_kbd_brightness_according_to_ambient_light (brightness, TRUE);
-}
-
-/**
- * gpm_brightness_kbd_has_hw:
- *
- * Self contained function that works out if we have the hardware.
- * If not, we return FALSE and the module is unloaded.
- **/
-gboolean
-gpm_brightness_kbd_has_hw (void)
-{
-	HalGManager *manager;
-	gchar **names;
-	gboolean ret = TRUE;
-
-	/* okay, as singleton - so we don't allocate more memory */
-	manager = hal_gmanager_new ();
-	hal_gmanager_find_capability (manager, "keyboard_backlight", &names, NULL);
+out:
+	hal_manager_free_capability (names);
 	g_object_unref (manager);
-
-	/* nothing found */
-	if (names == NULL || names[0] == NULL) {
-		ret = FALSE;
-	}
-
-	hal_gmanager_free_capability (names);
-	return ret;
 }
 
 /**
@@ -783,12 +769,6 @@ GpmBrightnessKbd *
 gpm_brightness_kbd_new (void)
 {
 	GpmBrightnessKbd *brightness;
-
-	/* only load an instance of this module if we have the hardware */
-	if (gpm_brightness_kbd_has_hw () == FALSE) {
-		return NULL;
-	}
-
 	brightness = g_object_new (GPM_TYPE_BRIGHTNESS_KBD, NULL);
 	return GPM_BRIGHTNESS_KBD (brightness);
 }
